package com.ruoyi.common.pay.ali.service;

import cn.hutool.json.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConfig;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.domain.AlipayTradeCreateModel;
import com.alipay.api.domain.AlipayTradeQueryModel;
import com.alipay.api.domain.AlipayTradeRefundModel;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradeCloseRequest;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.request.AlipayTradeQueryRequest;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradeCloseResponse;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.alipay.api.response.AlipayTradeQueryResponse;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.tm.IChannelInfo;
import com.ruoyi.common.pay.wx.Payable;
import com.ruoyi.common.pay.wx.RefundAble;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.Map;

/**
 * @author wjh
 * 2024/12/10
 */
@Service
public class AliPayService {

    // 证书模式
    public AlipayClient alipayClient(IChannelInfo channel) throws AlipayApiException {
        AlipayConfig alipayConfig = new AlipayConfig();
        //设置网关地址
        alipayConfig.setServerUrl("https://openapi.alipay.com/gateway.do");
        //设置应用APPID
        alipayConfig.setAppId(channel.getAppid());
        //设置应用私钥
        alipayConfig.setPrivateKey(channel.getPrivateKey());
        //设置应用公钥证书路径
        alipayConfig.setAppCertPath(channel.getAppCertPath());
        //设置支付宝公钥证书路径
        alipayConfig.setAlipayPublicCertPath(channel.getAlipayPublicCertPath());
        //设置支付宝根证书路径平台
        alipayConfig.setRootCertPath(channel.getRootCertPath());
        //设置请求格式，固定值json
        alipayConfig.setFormat("json");
        //设置字符集
        alipayConfig.setCharset("UTF-8");
        //设置签名类型
        alipayConfig.setSignType("RSA2");

        return new DefaultAlipayClient(alipayConfig);
    }

    public String pay(IChannelInfo channel,Payable payable) {
        // 构造请求参数以调用接口
        AlipayTradeCreateRequest request = new AlipayTradeCreateRequest();
        AlipayTradeCreateModel data = new AlipayTradeCreateModel();
        // 设置商户订单号
        data.setOutTradeNo(payable.getOutTradeNo());
        // 设置产品码
        data.setProductCode("JSAPI_PAY");
        // 设置小程序appId
        data.setOpAppId(channel.getAppid());
        // 设置订单总金额（元）
        data.setTotalAmount(String.valueOf(getPayableYuan(payable.getAmount())));
        // 设置订单标题
        data.setSubject(payable.getDescription());
        // 设置订单附加信息
        data.setBody(payable.getDescription());
        // 设置买家支付宝用户唯一标识
        data.setBuyerOpenId(payable.getOpenid());
        request.setBizModel(data);
        request.setNotifyUrl(channel.getNotifyUrl());

        try {
            AlipayClient alipayClient = alipayClient(channel);
            AlipayTradeCreateResponse response = alipayClient.certificateExecute(request);
            ServiceUtil.assertion(!response.isSuccess(), response.getSubMsg());
            return response.getTradeNo();
        } catch (Exception e) {
            throw new ServiceException("调用支付宝下单失败：" + e.getMessage());
        }
    }

    private BigDecimal getPayableYuan(long amount) {
        return new BigDecimal(amount).divide(BigDecimal.valueOf(100),2, RoundingMode.HALF_UP);
    }

//    /**
//     * 获取价格（元）
//     */
//    public BigDecimal payableYuan(long payFee) {
//        return new BigDecimal(payFee).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP);
//    }

    /**
     * 关闭支付订单
     */
    public void closeOrder(IChannelInfo channel,String outTradeNo) {
        AlipayTradeCloseRequest request = new AlipayTradeCloseRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("out_trade_no", outTradeNo);
        request.setBizContent(bizContent.toString());
        try {
            AlipayClient alipayClient = alipayClient(channel);
            AlipayTradeCloseResponse response = alipayClient.certificateExecute(request);
            ServiceUtil.assertion(!response.isSuccess(), response.getMsg());
        } catch (Exception e) {
            throw new ServiceException("调用支付宝关闭订单失败：" + e.getMessage());
        }
    }


    public AlipayTradeQueryResponse queryByOutTradeNo(IChannelInfo channel,String outTradeNo) {
        // 构造请求参数以调用接口
        AlipayTradeQueryRequest request = new AlipayTradeQueryRequest();
        AlipayTradeQueryModel model = new AlipayTradeQueryModel();

        // 设置订单支付时传入的商户订单号
        model.setOutTradeNo(outTradeNo);
        request.setBizModel(model);

        try {
            AlipayClient alipayClient = alipayClient(channel);
            AlipayTradeQueryResponse response = alipayClient.certificateExecute(request);
            ServiceUtil.assertion(!response.isSuccess(), response.getMsg());
            return response;
        } catch (Exception e) {
            throw new ServiceException("调用支付宝查询订单失败：" + e.getMessage());
        }
    }


    /**
     * 发起退款
     *
     * @param refund
     */
    public AlipayTradeRefundResponse refund(IChannelInfo channel, RefundAble refund) {
        // 构造请求参数以调用接口
        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
        AlipayTradeRefundModel model = new AlipayTradeRefundModel();

        // 设置商户订单号
        model.setOutTradeNo(refund.getOutTradeNo());
        // 设置退款金额
        model.setRefundAmount(String.valueOf(getPayableYuan(refund.getAmount())));
        // 设置退款原因说明
        model.setRefundReason(refund.getReason());
        // 设置退款请求号
        model.setOutRequestNo(refund.getOutRefundNo());
        request.setBizModel(model);
        request.setNotifyUrl(channel.getNotifyUrl());

        try {
            AlipayClient alipayClient = alipayClient(channel);
            AlipayTradeRefundResponse response = alipayClient.certificateExecute(request);
            ServiceUtil.assertion(!response.isSuccess(), response.getMsg());
            ServiceUtil.assertion(!"Y".equals(response.getFundChange()), "退款失败，本次退款未导致资金变化");
            return response;
        } catch (Exception e) {
            throw new ServiceException("调用支付宝退款失败：" + e.getMessage());
        }
    }

//    @Override
//    public boolean isPaySuccessByOutTradeNo(String outTradeNo) {
//        AlipayTradeQueryResponse res = this.queryByOutTradeNo(outTradeNo);
//        return isPaySuccess(res);
//    }
//
//    @Override
//    public boolean isPaySuccess(Object result) {
//        AlipayTradeQueryResponse res = (AlipayTradeQueryResponse) result;
//        return res != null && AliTradeStatusEnum.TRADE_SUCCESS.getStatus().equals(res.getTradeStatus());
//    }

    public LocalDateTime getPayTime(Object result) {
        AlipayTradeQueryResponse res = (AlipayTradeQueryResponse) result;
        return DateUtils.toLocalDateTime(res.getSendPayDate());
    }

    /**
     * 对通知进行验签
     */
    public void checkNotifySign(IChannelInfo channel, Map<String, String> params) {
        //调用SDK验证签名
        try {
            boolean signVerified = AlipaySignature.rsaCertCheckV1(params, channel.getAlipayPublicCertPath(),"UTF-8","RSA2");
            ServiceUtil.assertion(!signVerified, "验签失败");
        } catch (Exception e) {
            throw new ServiceException("验签失败" + e.getMessage());
        }
    }
}
